-- GLOBAL IGNORE LIST

----------------------------------
-- Global Ignore List Variables --
----------------------------------

local _, L          = ...
local safeToLoad    = false
local addonLoaded   = false
local serverName    = GetCVar("realmName");
local faction       = nil
local maxIgnoreSize = 50
local firstClear    = false
local firstPrune    = false
local pruneDays     = 0
local ignoreAdded   = false
local indentUI      = 0
local timer         = 0
local loadedTime    = GetTime()
local nameUI        = ""
 
local BlizzardAddIgnore        = nil
local BlizzardDelIgnore        = nil
local BlizzardAddOrDelIgnore   = nil
local BlizzardIgnoreListUpdate = nil

-------------
-- UNICODE --
-------------

local function GetWord (str, wordnumber)

	local len     = strlen(str)
	local count   = 1
	local res     = ""
	local words   = 0
	local nextInc = true
	local c
	
	while count <= len do
		if nextInc then
			words = words + 1
			
			if words > wordnumber then
				break
			end
		end
		
		c = strbyte(str, count)
		
		nextInc = (c == 32) and nextInc == false
		
		if words == wordnumber and c ~= 32 then
			res = res .. string.char(c)
		end
		
		count = count + 1
	end
	
	return res
end

local function Proper (name, okSpaces)

	if name == nil then return nil end

	local len    = strlen(name)
	local count  = 1
	local res    = ""
	local needUp = true
	local gotOP  = false
	local c
	
	while count <= len do
		c = strbyte(name, count)
		
		if c < 32 or c > 126 then
			res    = res .. string.char(c)
			needUp = false
		else
			if c ~= 32 or okSpaces == true then
				if needUp then
					res = res .. string.upper(string.char(c))
				else
					res = res .. string.lower(string.char(c))
				end
			
				gotOP  = (c == 40 or gotOP) and (c ~= 41)
				needUp = (c == 32 or c == 45 or gotOP == true)
			end
		end
		
		count  = count + 1
	end
	
	return res
end

local function strDown (str)

	local len    = strlen(str)
	local count  = 1
	local res    = ""
	local c
	
	while count <= len do
		c = strbyte(str, count)
		
		if c < 32 or c > 126 then
			res = res .. string.char(c)
		else
			res = res .. string.lower(string.char(c))
		end
		
		count = count + 1
	end
	
	return res
end

----------------------------------
-- Global Ignore List Functions --
----------------------------------

local function ShowMsg (msg)

	print ("|cff33ff99Global Ignore: |cffffffff" .. msg)

end

local function OnOff (value)

	if value == nil or value == false then
		return "|cffffff00" .. L["OFF"] .. "|cffffffff"
	elseif value == true then
		return "|cffffff00" .. L["ON"] .. "|cffffffff"
	else
		return "|cffff0000nil"
	end
end

local function dayString (value)

	if value == 1 then
		return L["DAY"]
	else
		return L["DAYS"]
	end
end

local function trim (str)

	local n = str:find"%S"
	return n and str:match(".*%S", n) or ""
 
end

local function dateToJulianDate (dateStr)
	
	if dateStr == nil then
		return 0
	end

	local monthList = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" }
	local words     = {}

	for word in dateStr:gmatch("%w+") do
		table.insert(words, word)
	end
	
	local day   = tonumber(words[1])
	local year  = tonumber(words[3])
	local month = 0

	for key, value in pairs(monthList) do
	
		if value == words[2] then
			month = tonumber(key)
			
			break
		end
	end

	--print ("monthStr="..monthStr.." month="..month.." day="..day.." year="..year)

	if (not month) or (not day) or (not year) or (month < 1) or (month > 12) or (day < 1) or (day > 31) or (year < 2014) then
		return 0
	end
	
	local calc1 = (month - 14) / 12
	local calc2 = day - 32075 + (1461 * (year + 4800 + calc1) / 4)
	
	calc2 = calc2 + (367 * (month - 2 - calc1 * 12) / 12)
	calc2 = calc2 - (3 * ((year + 4900 + calc1) / 100) / 4)
	
	return calc2
end

local function daysFromToday (dateStr)

	local addDate = dateToJulianDate(dateStr)
	local today   = dateToJulianDate(date("%d %b %Y"))
	
	if addDate == 0 then
		return -1
	else
	
		if addDate < today then
			return math.floor(today - addDate)
		else
			return math.floor(addDate - today)
		end
	end
end

local function AddToList(newname, newfaction, newnote, newtype)

	local index = #GlobalIgnoreDB.ignoreList+1
	
	GlobalIgnoreDB.ignoreList[index] = newname
	GlobalIgnoreDB.factionList[index] = newfaction
	GlobalIgnoreDB.dateList[index] = date("%d %b %Y")
	GlobalIgnoreDB.notes[index] = (newnote or "")
	GlobalIgnoreDB.expList[index] = GlobalIgnoreDB.defexpire
	GlobalIgnoreDB.typeList[index] = (newtype or "player")
	
end

local function RemoveFromList(index)

	if index <= #GlobalIgnoreDB.ignoreList then
		table.remove(GlobalIgnoreDB.ignoreList, index)
		table.remove(GlobalIgnoreDB.factionList, index)
		table.remove(GlobalIgnoreDB.dateList, index)
		table.remove(GlobalIgnoreDB.notes, index)
		table.remove(GlobalIgnoreDB.expList, index)
		table.remove(GlobalIgnoreDB.typeList, index)
	end
end

local function addServer (name)

	local result = name

	if string.find(name, "%-") == nil then
		result = name .. "-" .. serverName
	end
	
	return result
end

local function getServer (name)

	result = serverName

	local index = string.find(name, "%-");
	
	if index ~= nil then
		result = string.sub(name, index + 1, string.len(name));
	end
	
	return result
end

local function removeServer (name, strict)

	result = name	
	
	local index = string.find(name, "%-");
	
	if strict == nil then
		strict = false
	end
	
	if index ~= nil then
		local server = Proper(string.sub(name, index + 1, string.len(name)));
		
		if server == Proper(serverName) or strict == true then
			result = string.sub(name, 1, index - 1)
		end
	end
	
	return result	
end

local function isServerMatch (server1, server2)

	return Proper(server1) == Proper(server2)
	
end

local function hasIgnored (name)

	local result = 0
	
	name = Proper(addServer(name))
		
	for count = 1, GetNumIgnores() do
	
		if name == Proper(addServer(GetIgnoreName(count))) then
			result = count
			
			break
		end
	end
	
	return result
end

local function hasNPCignored (name)

	local result = 0
	
	if not name then return result end
	
	name = Proper(name, true)
	
	for key, value in pairs(GlobalIgnoreDB.ignoreList) do
	
		if value == name and GlobalIgnoreDB.typeList[key] == "npc" then
			result = key
			
			break
		end
	end
	
	return result
end

local function hasServerIgnored (name)

	local result = 0
	
	if not name then return result end
	
	name = Proper(name, false)
	
	for key, value in pairs(GlobalIgnoreDB.ignoreList) do
	
		if value == name and GlobalIgnoreDB.typeList[key] == "server" then
			result = key
			
			break
		end
	end
	
	return result
end

local function hasGlobalIgnored (name)
	
	local result = 0
	
	if not name then return result end
	
	name = Proper(addServer(name))
	
	for key,value in pairs(GlobalIgnoreDB.ignoreList) do
	
		if value == name and GlobalIgnoreDB.typeList[key] == "player" then
			result = key
			
			break
		end
	end
	
	return result
end

local function ShowIgnoreList (param)

	local days   = tonumber(param)
	local sName  = ""
	
	if not days then
	
		days  = 0
		sName = param
		
		if not sName then
			sName = ""
		end
	end
	
	if days > 0 then
		ShowMsg("|cffffff00" .. format(L["LIST_1"], days))		
	else
		if sName ~= "" then
			sName = Proper(sName)
			
			if sName == "Npc" then
				ShowMsg("|cffffff00".. L["LIST_2"])
			else			
				if sName == "Server" then
					sName = serverName		
				end
				
				ShowMsg("|cffffff00" .. format(L["LIST_3"], sName))
			end
		else
			ShowMsg("|cffffff00" .. L["LIST_4"])
		end
	end
	
	local count = 0
	
	for key,value in pairs(GlobalIgnoreDB.ignoreList) do
	
		local ok   = true
		local type = "P"
		
		if GlobalIgnoreDB.typeList[key] == "npc" then
			type = "N"
		end
	
		if days > 0 then
		
			ok = daysFromToday(GlobalIgnoreDB.dateList[key]) >= days
			
		elseif sName ~= "" then
		
			ok = (type == "N" and sName == "Npc") or (type == "P" and isServerMatch(Proper(sName), getServer(value)))

		end
		
		if ok then
			local str = "  (" .. key .. ") [" .. type.. "] " .. value .. " (" .. (GlobalIgnoreDB.factionList[key] or "Unknown") .. ") " .. "[".. daysFromToday(GlobalIgnoreDB.dateList[key]) .. " "..L["DAYS"] .. "]"
			
			if GlobalIgnoreDB.notes[key] ~= "" then
				str = str .." (" .. GlobalIgnoreDB.notes[key] .. ")"
			end
			
			ShowMsg(str)

			count = count + 1			
		end
		
	end
	
	ShowMsg("|cffffff00" .. format(L["LIST_5"], count))
end

local function ResetIgnoreDB()

	GlobalIgnoreDB = {
		chatmsg     = true,
		sameserver  = true,
		samefaction = true,
		chatfilter  = true,
		defexpire   = 0,
		ignoreList  = {},
		factionList = {},
		dateList    = {},
		notes       = {},
		expList     = {},
		typeList    = {}
	}
	
	GlobalIgnoreImported = false
end

local function SyncIgnoreList(silent)

	if silent == nil then
		silent = false
	end
	
	for count = 1, GetNumIgnores() do
	
		local name = addServer(GetIgnoreName(count))
	
		if hasGlobalIgnored(name) == 0 then
			name = removeServer(name)
			
			if not silent then
				ShowMsg (format(L["SYNC_1"], name))
			end			
			
			BlizzardDelIgnore (hasIgnored(name))
		end
	end
	
	local count = 0
	
	while count < #GlobalIgnoreDB.dateList do
		count = count + 1
		
		if GlobalIgnoreDB.expList[count] > 0 and daysFromToday(GlobalIgnoreDB.dateList[count]) >= GlobalIgnoreDB.expList[count] then
			local name = addServer(GlobalIgnoreDB.ignoreList[count])		
			DelIgnore(name)
			count = 0
		end
	end
	
	for key,value in pairs(GlobalIgnoreDB.ignoreList) do
	
		if GlobalIgnoreDB.typeList[key] == "player" then
		
			local name = addServer(value)
			
			if hasIgnored(name) == 0 then
		
				local ok = (GlobalIgnoreDB.factionList[key] == faction)
			
				if ok then		
					ok = (isServerMatch(serverName, getServer(name))) or (GlobalIgnoreDB.sameserver == false)				
				end
			
				if ok then
					if GetNumIgnores() < maxIgnoreSize then
						name = removeServer(name)
			
						if not silent then
							ShowMsg (format(L["SYNC_2"], name))
						end
			
						BlizzardAddIgnore(name)
					end
				end
			end
		end
	end	
end

local function PruneIgnoreList (days, doit)

	if days == nil or days <= 0 then		
		return 0
	end

	local targets = 0
	local count   = 0
	
	while count < #GlobalIgnoreDB.dateList do
		count = count + 1
	
		if daysFromToday(GlobalIgnoreDB.dateList[count]) >= days then
			targets = targets + 1
			
			local name = addServer(GlobalIgnoreDB.ignoreList[count])
					
			if doit == true then
				DelIgnore(name)
				count = 0
			end
		end
	end
	
	return targets
end

local function ApplicationStartup(self)

	if addonLoaded or safeToLoad == false then
		return
	end
	
	ShowMsg(L["LOAD_1"])
		
	faction = UnitFactionGroup("player")
		
	if GlobalIgnoreDB == nil then
		
		ResetIgnoreDB()	
	end
	
	-- set missing defaults or upgrade if needed
		
	if not GlobalIgnoreDB.defexpire then
		GlobalIgnoreDB.defexpire = 0
	end
	
	if GlobalIgnoreDB.asknote == nil then
		GlobalIgnoreDB.asknote = true
	end
		
	if not GlobalIgnoreDB.expList then
		GlobalIgnoreDB.expList = {}
			
		for  count = 1, #GlobalIgnoreDB.ignoreList do
			GlobalIgnoreDB.expList[count] = 0
		end
	end
	
	if not GlobalIgnoreDB.typeList then
		GlobalIgnoreDB.typeList = {}
		
		for count = 1, #GlobalIgnoreDB.ignoreList do
			GlobalIgnoreDB.typeList[count] = "player"
		end
	end
	
	if GlobalIgnoreDB.revision == nil then
		GlobalIgnoreDB.revision = 1
		
		for count = 1, #GlobalIgnoreDB.ignoreList do
			GlobalIgnoreDB.ignoreList[count] = Proper(GlobalIgnoreDB.ignoreList[count])
		end
	end
	
	loadedTime = GetTime()
		
	--
		
	if GlobalIgnoreImported == nil or GlobalIgnoreImported ~= true then
	
		local ignores = GetNumIgnores()	
		local added   = 0
			
		if (ignores > 0) and (GlobalIgnoreDB.chatmsg == nil or GlobalIgnoreDB.chatmsg == true) then
			ShowMsg(L["LOAD_2"])
		end
			
		for count = 1, ignores do
			
			local name = GetIgnoreName(count)
				
			if removeServer(name, true) ~= "Unknown" then
				
				name = Proper(addServer(name))

				if hasGlobalIgnored(name) == 0 then
					added = added + 1
						
					AddToList(name, faction)
										
					if GlobalIgnoreDB.chatmsg == nil or GlobalIgnoreDB.chatmsg == true then
						ShowMsg (format(L["LOAD_3"], name))
					end
				end
			end
				
		end
			
		GlobalIgnoreImported = true
	end
						
	-- Check for GlobalIgnoreDB nil values and set defaults here if needed
		
	if GlobalIgnoreDB.chatfilter == nil then
		GlobalIgnoreDB.chatfilter = true
	end

	SyncIgnoreList(GlobalIgnoreDB.chatmsg == nil or GlobalIgnoreDB.chatmsg == false)

	if GetNumIgnores() >= maxIgnoreSize then
		ShowMsg(format(L["LOAD_4"], maxIgnoreSize))		
	end
		
	addonLoaded = true
	
	self:UnregisterEvent("IGNORELIST_UPDATE")
end

local function EventHandler (self, event, sender, ...)

	--print ("event=".. (event or "nil"))
	
	if event == "IGNORELIST_UPDATE" then
		safeToLoad = true
	elseif event == "PLAYER_LOGIN" then
		if not safeToLoad and not addonLoaded then
		
			timer = 0
		
			self:SetScript("OnUpdate", function(Self, elapsed)
			
				timer = timer + elapsed
							
				if timer < 3 then
					return
				end
												
				self:SetScript("OnUpdate", nil)
				self:Hide()
				
				safeToLoad = true
				
				ApplicationStartup(self)
			end)
		end
	end
	
	if safeToLoad == true then
		ApplicationStartup(self)
	end
end

function SlashCmdList.GIGNORE (msg)

	msg = strDown(msg)
	
	local args   = {}
	local argStr = ""
	local count  = 1

	local str = GetWord(msg, count)
	
	while str ~= "" do
		
		table.insert(args, str)
		
		if count == 2 then
			argStr = str
		elseif count > 2 then
			argStr = argStr .. " ".. str
		end
		
		count = count + 1
		str   = GetWord(msg, count)
	end
	
	if args[1] == "clear" then
	
		if firstClear and args[2] ~= nil and args[2] == "confirm" then	
			ResetIgnoreDB()
			ShowMsg(L["CMD_2"])
			SyncIgnoreList(GlobalIgnoreDB.chatmsg == nil or GlobalIgnoreDB.chatmsg == false)
			firstClear = false
		else
			ShowMsg("|cffff0000" .. L["CMD_1"])
			firstClear = true
		end
		
	elseif args[1] == "defexpire" then
	
		if tonumber(args[2]) then
			GlobalIgnoreDB.defexpire = tonumber(args[2])
			
			ShowMsg (format(L["CMD_3"], GlobalIgnoreDB.defexpire, dayString(GlobalIgnoreDB.defexpire)))
		end
		
	elseif msg == "asknote true" or msg == "asknote on" then
	
		GlobalIgnoreDB.asknote = true
		ShowMsg (L["CMD_4"])

	elseif msg == "asknote false" or msg == "asknote off" then

		GlobalIgnoreDB.asknote = false
		ShowMsg (L["CMD_5"])
		
	elseif msg == "showmsg true" or msg == "showmsg on" then
	
		GlobalIgnoreDB.chatmsg = true
		ShowMsg (L["CMD_6"])

	elseif msg == "showmsg false" or msg == "showmsg off" then
	
		GlobalIgnoreDB.chatmsg = false	
		ShowMsg (L["CMD_7"])
	
	elseif msg == "filter true" or msg == "filter on" then

		GlobalIgnoreDB.chatfilter = true
		ShowMsg(L["CMD_8"])

	elseif msg == "filter false" or msg == "filter off" then

		GlobalIgnoreDB.chatfilter = false
		ShowMsg(L["CMD_9"])
		
	elseif msg == "server true" or msg == "server on" then
	
		GlobalIgnoreDB.sameserver = true
		ShowMsg(L["CMD_10"])

	elseif msg == "server false" or msg == "server off" then
	
		GlobalIgnoreDB.sameserver = false
		ShowMsg(L["CMD_11"])
		
	elseif args[1] == "list" then
	
		ShowIgnoreList(argStr)
		
	elseif (args[1] == "add" or args[1] == "ignore") then
	
		argStr = (trim(Proper(argStr)) or "")
		
		if argStr == "" then
			argStr, server =  UnitName("target")
					
			if server ~= nil then
				argStr = argStr .. "-"..server
			end
			
			if argStr == nil or not UnitPlayerControlled("target") then
				argStr = ""
			end
		end
		
		if argStr ~= "" then
			AddIgnore (Proper(argStr))
		end
		
	elseif (args[1] == "remove" or args[1] == "delete") and args[2] ~= nil and args[2] ~= "" then
	
		if tonumber(argStr) then
		
			local str = GlobalIgnoreDB.typeList[tonumber(argStr)]
			
			if str == "npc" then
				ShowMsg (format(L["CMD_12"], Proper(GlobalIgnoreDB.ignoreList[tonumber(argStr)], true)))
				RemoveFromList(tonumber(argStr))
			else
				DelIgnore(args[2], true)
			end
		else
			local npcIndex = hasNPCignored(argStr)
		
			if npcIndex > 0 then
				ShowMsg (format(L["CMD_12"], Proper(argStr, true)))
				RemoveFromList(npcIndex)
			else	
				DelIgnore (args[2], true)
			end
		end

	elseif (args[1] == "npc" or args[1] == "addnpc") then
	
		if not tonumber(argStr) then
				
			argStr = (trim(Proper(argStr, true)) or "")
	
			if argStr == "" then
				argStr = Proper(UnitName("target"), true)
				
				if argStr == nil or UnitPlayerControlled("target") then
					argStr = ""
				end
			end
			
			if argStr ~= "" then

				local npcIndex = hasNPCignored(argStr)
		
				if npcIndex > 0 then
					local name = GlobalIgnoreDB.ignoreList[npcIndex]
			
					ShowMsg (format(L["CMD_12"], name))
					RemoveFromList(npcIndex)
				else
					ShowMsg (format(L["CMD_13"], argStr))
					AddToList(argStr, faction, "", "npc")
				end
			end
		end
		
	elseif args[1] == "expire" and args[2] ~= nil and args[2] ~= "" and tonumber(args[3]) then

		if tonumber(args[2]) then

			local index = tonumber(args[2])

			if (index > 0) and (index <= #GlobalIgnoreDB.ignoreList) then
			
				GlobalIgnoreDB.expList[index] = tonumber(args[3])
				ShowMsg(format(L["CMD_14"], GlobalIgnoreDB.ignoreList[index], tonumber(args[3])))
			end

		else

			local playerIndex = hasGlobalIgnored(args[2])

			if playerIndex > 0 then
				GlobalIgnoreDB.expList[playerIndex] = tonumber(args[3])
				ShowMsg(format(L["CMD_14"], Proper(args[2]), tonumber(args[3])))
			end
		end
		
	--elseif args[1] == "gui" then	
	--	GUI()
		
	elseif args[1] == "sync" then
	
		SyncIgnoreList()
		
	elseif args[1] == "prune" then
	
		if args[2] == "confirm" and firstPrune == true then
		
			PruneIgnoreList(pruneDays, true)
			
			firstPrune = false
		elseif args[2] == nil or tonumber(args[2]) == nil then
		
			ShowMsg(L["CMD_15"])
		else
			if firstPrune == false then

				pruneDays = tonumber(args[2])
				
				ShowMsg(format(L["CMD_16"], pruneDays))
				ShowMsg(format(L["CMD_17"], PruneIgnoreList(pruneDays, false)))
				
				firstPrune = true
			end
		end

	else
		ShowMsg (L["HELP_1"])
		ShowMsg ("")
		ShowMsg ("  " .. L["HELP_2"])
		ShowMsg ("  " .. L["HELP_3"])
		ShowMsg ("  " .. L["HELP_4"])
		ShowMsg ("  " .. L["HELP_5"])
		ShowMsg ("  " .. L["HELP_6"])
		ShowMsg ("  " .. L["HELP_7"])
		ShowMsg ("  " .. L["HELP_8"])	
		ShowMsg ("")
		ShowMsg ("  " .. format(L["HELP_9"], OnOff(GlobalIgnoreDB.chatfilter)))
		ShowMsg ("  " .. format(L["HELP_10"], OnOff(GlobalIgnoreDB.chatmsg)))
		ShowMsg ("  " .. format(L["HELP_11"], OnOff(GlobalIgnoreDB.sameserver)))
		ShowMsg ("  " .. format(L["HELP_12"], GlobalIgnoreDB.defexpire))
		ShowMsg ("  " .. format(L["HELP_13"], OnOff(GlobalIgnoreDB.asknote)))
		ShowMsg ("")
		ShowMsg (L["HELP_14"])
	end
end

-----------------------------
-- Global Ignore List Main --
-----------------------------

local frame = CreateFrame("FRAME")

frame:RegisterEvent("IGNORELIST_UPDATE")
frame:RegisterEvent("PLAYER_LOGIN")

SLASH_GIGNORE1 = "/gignore"
SLASH_GIGNORE2 = "/gi"
SLASH_GIGNORE3 = "/gil"

frame:SetScript("OnEvent", EventHandler)

------------------------
-- Function Pre Hooks --
------------------------

BlizzardAddIgnore      = AddIgnore
BlizzardDelIgnore      = DelIgnore
BlizzardAddOrDelIgnore = AddOrDelIgnore

AddIgnore = function(name)

	if (not name or name == "") then

		name, server = UnitName("target")
					
		if server ~= nil then
			name = name .. "-"..server
		end
	end
	
	if (not name or name == "") then
		return
	end

	local note  = ""
	local space = string.find(name, " ")
	local days  = GlobalIgnoreDB.defexpire
	
	if space then
		note = string.sub(name, space + 1)
		name = string.sub(name, 0, space - 1)
		
		space = string.find(note, " ")
		
		if space then
			if tonumber(string.sub(note, 0, space - 1)) then
				days = tonumber(string.sub(note, 0, space - 1))
				note = string.sub(note, space + 1)
			end
		else
			if tonumber(note) then
				days = tonumber(note)
				note = ""
			end
		end
	end
	
	name = Proper(addServer(name))
	
	if Proper(addServer(UnitName("player"))) ~= name then

		local index = hasGlobalIgnored(name)
	
		if index == 0 then
			AddToList(name, faction, note)
			
			if GlobalIgnoreDB.asknote == true then
			
				nameUI = name				
				StaticPopup_Show("GIL_REASON", nameUI)			
			end
			
			ShowMsg(format(L["ADD_2"], name))
			
			if GetNumIgnores() < maxIgnoreSize then
				BlizzardAddIgnore(removeServer(name))
			end
		else
			if hasIgnored(name) > 0 then
				ShowMsg(format(L["ADD_1"], name))
			end
		
			BlizzardAddIgnore(removeServer(name))		
		end
		
		indentUI = 0

	else
		ShowMsg(L["ADD_3"])
	end	
end

DelIgnore = function(idxpos, isGIL)

	local name = ""
	
	if isGIL then
		if tonumber(idxpos) ~= nil then
			name = GlobalIgnoreDB.ignoreList[tonumber(idxpos)]
			
			if name == nil then
				return
			end
		else
			name = idxpos
		end
	else
		if tonumber(idxpos) ~= nil then			
			name = GetIgnoreName(idxpos)
		else
			name   = idxpos
			idxpos = hasIgnored(name)
		end
	end
	
	name = Proper(addServer(name))
	
	local index = hasGlobalIgnored(name)

	if index > 0 then
		ShowMsg(format(L["REM_1"], name))
		
		RemoveFromList(index)		
		
		BlizzardDelIgnore(removeServer(name))
	else
		BlizzardDelIgnore(idxpos)
	end
	
	indentUI = 0
end

AddOrDelIgnore = function(name)

	if (not name or name == "") then

		name, server = UnitName("target")
					
		if server ~= nil then
			name = name .. "-"..server
		end
	end
	
	if (not name or name == "") then
		return
	end
	
	local index = hasGlobalIgnored(name)

	if index == 0 then
		AddIgnore(name)
	else
		DelIgnore(index, true)
	end	
end

--------------------
-- Chat Filtering --
--------------------

local function chatMessageFilter (self, event, message, from, ...)

	--print ("chatMsg evt=" .. (event or "nil") .. " msg=".. (message or "nil") .. " from=" .. (from or "nil"))
	
	if addonLoaded ~= true then
		return false
	end
	
	if GlobalIgnoreDB.chatfilter == true then
	
		if event == "CHAT_MSG_MONSTER_EMOTE" or event == "CHAT_MSG_MONSTER_PARTY" or event == "CHAT_MSG_MONSTER_SAY" or
		   event == "CHAT_MSG_MONSTER_WHISPER" or event == "CHAT_MSG_MONSTER_YELL" then

	        	return (hasNPCignored(from) > 0)
			
		elseif event == "CHAT_MSG_SYSTEM" then
		
			if message == ERR_IGNORE_NOT_FOUND and GetTime() - loadedTime < 60 then
				return true			
			end
			
			if message == ERR_IGNORE_FULL then
				return true
			end
	
		  	for key, value in pairs(GlobalIgnoreDB.ignoreList) do
		  	
		  		if isServerMatch(serverName, getServer(value)) then
		  		
		  			local pName      = removeServer(value)
					local msgOffline = string.format(ERR_FRIEND_OFFLINE_S, pName)
					local msgOnline  = string.format(ERR_FRIEND_ONLINE_SS, pName, pName)
					
					if (message == msgOffline) or (message == msgOnline) then
						return true
					end
		  		end		  		
		  	end
		  	
		elseif (from ~= nil) and (from ~= "") then
		
			if hasGlobalIgnored(from) > 0 then
				return true
			else
				return false
			end
		end		  	
	end
	
	return false
end

local chatEvents = (
		{
		"CHAT_MSG_ACHIEVEMENT",
		"CHAT_MSG_BATTLEGROUND",
		"CHAT_MSG_BATTLEGROUND_LEADER",
		"CHAT_MSG_CHANNEL",
		"CHAT_MSG_CHANNEL_JOIN",
		"CHAT_MSG_CHANNEL_LEAVE",
		"CHAT_MSG_EMOTE",
		"CHAT_MSG_GUILD",
		"CHAT_MSG_GUILD_ACHIEVEMENT",	
		"CHAT_MSG_MONSTER_EMOTE",
		"CHAT_MSG_MONSTER_PARTY",
		"CHAT_MSG_MONSTER_SAY",
		"CHAT_MSG_MONSTER_WHISPER",
		"CHAT_MSG_MONSTER_YELL",
		"CHAT_MSG_OFFICER",
		"CHAT_MSG_PARTY",
		"CHAT_MSG_RAID",
		"CHAT_MSG_RAID_LEADER",
		"CHAT_MSG_RAID_WARNING",
		"CHAT_MSG_SAY",
		"CHAT_MSG_SYSTEM",
		"CHAT_MSG_TEXT_EMOTE",
		"CHAT_MSG_WHISPER",
		"CHAT_MSG_WHISPER_INFORM",
		"CHAT_MSG_YELL"
		}
	)

for key, value in pairs (chatEvents) do
	ChatFrame_AddMessageEventFilter(value, chatMessageFilter)
end

--------------
-- UI Hacks --
--------------

local daysUI   = {}
local expUI    = {}
local reasonUI = {}
local headerUI = {}

BlizzardIgnoreListUpdate = IgnoreList_Update

StaticPopupDialogs["GIL_REASON"] = {

  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text           = "Edit ignore note for |cffffff00%s:",
  maxLetters     = 128,
  hasEditBox	 = 1,
  whileDead      = 1,
  button1        = "Save",
  button2        = "Cancel",
  OnShow         = function(self)
			self.editBox:SetText(GlobalIgnoreDB.notes[hasGlobalIgnored(nameUI)])
			self.editBox:SetFocus()
  		   end,
  OnAccept       = function(self)
			GlobalIgnoreDB.notes[hasGlobalIgnored(nameUI)] = self.editBox:GetText()
			IgnoreList_Update()  			
                   end,
  EditBoxOnEnterPressed = function(self)
  
  				GlobalIgnoreDB.notes[hasGlobalIgnored(nameUI)] = self:GetParent().editBox:GetText()
  				IgnoreList_Update()
  				self:GetParent():Hide()
  			end,
  EditBoxOnEscapePressed = function(self)
  				self:GetParent():Hide()
  			end
}

function setExpire (text)

	if tonumber(text) then
		GlobalIgnoreDB.expList[hasGlobalIgnored(GetIgnoreName(GetSelectedIgnore()))] = tonumber(text)
	
		IgnoreList_Update()
	end	
end

StaticPopupDialogs["GIL_EXPIRE"] = {

  preferredIndex = STATICPOPUPS_NUMDIALOGS,
  text           = "|cffffff00%s|cffffffff\n\n"..L["BOX_1"],
  maxLetters     = 4,
  hasEditBox	 = 1,
  whileDead      = 1,
  button1        = "Save",
  button2        = "Cancel",
  OnShow         = function(self)
  			local playerIndex = hasGlobalIgnored(GetIgnoreName(GetSelectedIgnore()))
  			
  			self.editBox:SetText(GlobalIgnoreDB.expList[playerIndex])
  			self.editBox:SetFocus()
  		   end,
  OnAccept       = function(self)
  			setExpire(self.editBox:GetText())
                   end,
  EditBoxOnEnterPressed = function(self)
  				setExpire(self:GetParent().editBox:GetText())
  				self:GetParent():Hide()
  			end,
  EditBoxOnEscapePressed = function(self)
  				self:GetParent():Hide()
  			end
}

function IgnoreButtonEnter(self, ...)

	GameTooltip:Hide()
	GameTooltip:ClearLines()
	
	local playerIndex = hasGlobalIgnored(self.name:GetText())
	
	if playerIndex == 0 then
		return
	end
	
	local playerNotes = GlobalIgnoreDB.notes[playerIndex]
	local daysInList  = daysFromToday(GlobalIgnoreDB.dateList[playerIndex])
	local daysExpire  = GlobalIgnoreDB.expList[playerIndex]
		
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:AddLine("|cff69CCF0" .. self.name:GetText())
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine("  "..L["OVER_1"].." |cffc0c0c0" .. getServer(self.name:GetText()))
	GameTooltip:AddLine("  "..L["OVER_2"].." |cffc0c0c0" .. (GlobalIgnoreDB.factionList[playerIndex] or "Unknown"))
	GameTooltip:AddLine("  "..L["OVER_3"].." |cffc0c0c0" .. daysInList .. " " .. dayString(daysInList))
	
	if GlobalIgnoreDB.expList[playerIndex] == 0 then
		GameTooltip:AddLine("  "..L["OVER_4"].." |cffc0c0c0"..L["EXP_NVR"])
	else
		GameTooltip:AddLine("  "..L["OVER_4"].." |cffc0c0c0"..format(L["OVER_5"], daysExpire , daysExpire - daysInList))
	end
	
	GameTooltip:AddLine(" ")
	
	if playerNotes ~= nil and playerNotes ~= "" then
		GameTooltip:AddLine("|cff69CCF0"..playerNotes)
	end
	
	GameTooltip:Show()
end

function IgnoreButtonLeave(self, ...)
	GameTooltip:Hide()
end

function IgnoreButtonDoubleClick(self, ...)
	nameUI = GetIgnoreName(GetSelectedIgnore())
	StaticPopup_Show("GIL_REASON", nameUI)
end

function IgnoreButtonClick(self, button, down)

	if button ~= "RightButton" or down == true then
		return
	end
	
	local IgnoreRightClickMenu = {

		{ text = GetIgnoreName(GetSelectedIgnore()), isTitle = true, notCheckable = true },
		{ text = L["RCM_1"], notCheckable = true, func = function() IgnoreButtonDoubleClick() end },
		{ text = "", disabled = true, notCheckable = true },
		{ text = L["RCM_2"], notCheckable = true, func = function() StaticPopup_Show("GIL_EXPIRE", GetIgnoreName(GetSelectedIgnore())) end },
		{ text = L["RCM_3"], notCheckable = true, func = function() GlobalIgnoreDB.expList[hasGlobalIgnored(GetIgnoreName(GetSelectedIgnore()))] = 0 IgnoreList_Update() end },
		{ text = "", disabled = true, notCheckable = true },
		{ text = L["RCM_4"], notCheckable = true, func = function() DelIgnore(GetIgnoreName(GetSelectedIgnore())) end },
		{ text = "", notCheckable = true, disabled = true },
		{ text = L["RCM_5"], notCheckable = true }
	}	
	
	local menuFrame = CreateFrame("Frame", "gil_IgnoreRightClick", UIParent, "UIDropDownMenuTemplate")

	EasyMenu(IgnoreRightClickMenu, menuFrame, "cursor", 0 , 0, "MENU")
end

IgnoreList_Update = function(...)

	BlizzardIgnoreListUpdate(...)

	if addonLoaded ~= true then
		return
	end
	
	local buttonWidth = 0
	
	if indentUI == 0 then
		for count = 1, IGNORES_TO_DISPLAY, 1 do
			local button = _G["FriendsFrameIgnoreButton"..count]
		
			if button then
				indentUI = max(indentUI, button.name:GetStringWidth())
			end
		end
	end

	for count = 1, IGNORES_TO_DISPLAY, 1 do
		button = _G["FriendsFrameIgnoreButton"..count]
		
		if button then

			if buttonWidth == 0 then
				buttonWidth = button:GetWidth()
			end
			
			if daysUI[count] == nil then		
				daysUI[count] = button:CreateFontString("FontString", "OVERLAY", "GameFontWhiteSmall")
		
				daysUI[count]:SetPoint("TOP", button)
				daysUI[count]:SetPoint("BOTTOMLEFT", button)
				daysUI[count]:SetJustifyH("RIGHT")
					
				button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
				
				button:HookScript("OnEnter", IgnoreButtonEnter)
				button:HookScript("OnLeave", IgnoreButtonLeave)
				button:HookScript("OnClick", IgnoreButtonClick)
				button:HookScript("OnDoubleClick", IgnoreButtonDoubleClick)
			end
			
			if expUI[count] == nil then		
				expUI[count] = button:CreateFontString("FontString", "OVERLAY", "GameFontWhiteSmall")
					
				expUI[count]:SetPoint("TOP", button)
				expUI[count]:SetPoint("BOTTOMLEFT", button)
				expUI[count]:SetJustifyH("RIGHT")
			end
			
			if reasonUI[count] == nil then
				reasonUI[count] = button:CreateFontString("FontString", "OVERLAY", "GameFontWhiteSmall")
		
				reasonUI[count]:SetPoint("TOP", button)
				reasonUI[count]:SetPoint("BOTTOMRIGHT", button)
				reasonUI[count]:SetJustifyH("LEFT")
			end				

			daysUI[count]:SetWidth(indentUI + 50)
			expUI[count]:SetWidth(indentUI + 90)
			reasonUI[count]:SetWidth(buttonWidth - indentUI - 100)
			
			if count == 1 and FauxScrollFrame_GetOffset(FriendsFrameIgnoreScrollFrame) == 0 then
			
				if headerUI[1] == nil then
					headerUI[1] = button:CreateFontString("FontString", "OVERLAY", "GameFontWhiteSmall")
					
					headerUI[1]:SetPoint("TOP", button)
					headerUI[1]:SetPoint("BOTTOMLEFT", button)
					headerUI[1]:SetPoint("LEFT", 58, 0)
					headerUI[1]:SetJustifyH("RIGHT")	
				end
				
				local ignoreNum = GetNumIgnores()
				
				if ignoreNum == maxIgnoreSize then			
					headerUI[1]:SetText("["..ignoreNum.."|cff00ff96+" .. (#GlobalIgnoreDB.ignoreList - ignoreNum) .."|cffffffff]")
				else
					headerUI[1]:SetText("["..ignoreNum.."]")
				end
								
				daysUI[1]:SetText("Days")
				expUI[1]:SetText("Expire")
				reasonUI[1]:SetText("Note")
			
			else
			
				if count == 1 then
					headerUI[1]:SetText("")
				end
				
				playerIndex = hasGlobalIgnored(button.name:GetText())
				
				if playerIndex > 0 then			
					daysUI[count]:SetText(daysFromToday(GlobalIgnoreDB.dateList[playerIndex]) .. "d")
				
					local playerExp = (GlobalIgnoreDB.expList[playerIndex] or 0)
					local daysExp   = playerExp - daysFromToday(GlobalIgnoreDB.dateList[playerIndex])
				
					if playerExp == 0 then
						expUI[count]:SetText("|cff808080"..L["EXP_NVR"])
					elseif daysExp <= 0 then
						expUI[count]:SetText("|cffff6666"..L["EXP_TDY"])
					else
						expUI[count]:SetText(daysExp.."d")
					end
					
					reasonUI[count]:SetText("|cff69CCF0"..GlobalIgnoreDB.notes[playerIndex])
				end
			end				
		end
	end


end

local function GilUnitMenu (dropdownMenu, which, unit, name, userData, ...)

	if (UIDROPDOWNMENU_MENU_LEVEL > 1) then
		return
	end
	
	if (which and (which == "PLAYER" or which == "RAID_PLAYER" or which == "PARTY")) then
	
		local target, server =  UnitName("target")
			
		--print ("DEBUG target="..(target or "Unknown").. " server="..(server or "Unknown"));
					
		if server then
			target = target .. "-"..server
		end
		
		DropDownList1.numButtons = max(0, DropDownList1.numButtons - 1)

		local info = UIDropDownMenu_CreateInfo()
		info.text = ""
		info.notCheckable = true
		info.disabled = true
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)	

		local info = UIDropDownMenu_CreateInfo()
				
		info.dist = 0
		info.func = function() AddOrDelIgnore("") end
		info.notCheckable = 1
		
		if (hasGlobalIgnored(target) > 0) then
			info.text = "Remove from Global Ignore"
		else
			info.text = "Add to Global Ignore"
		end
		
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)
		
		local info = UIDropDownMenu_CreateInfo()
		info.text = "Cancel"
		info.dist = 0
		info.notCheckable = 1
		UIDropDownMenu_AddButton(info, UIDROPDOWNMENU_MENU_LEVEL)
	end
end

hooksecurefunc("UnitPopup_ShowMenu", GilUnitMenu)

-----------------
-- FUTURE GUI ---
-----------------

-- Name                    Server               Faction   Listed  Expire  Note
-- Superlongnam            The Forgotten Coast  Alliance  1234d   Never   Note goes here and stuff
-- Snurk Bucksquick Name   *                    NPC       1234d   Never   Note goes here and stuff

local BUTTON_TOTAL  = 5
local BUTTON_HEIGHT = 20
local BUTTON_WIDTH  = 560

local list        = {"Superlongnam             The Forgotten Coast  Alliance  1234d  Never  Note goes here and stuff", "Snurk Bucksquick's Name  *                         NPC  1234d  Never  Note goes here and stuff", "aaa", "bbb", "ccc", "ddd", "eee", "fff", "ggg", "hhh"}
local buttons     = {}
local scrollFrame = nil

local function GIL_UpdateList (self)

	FauxScrollFrame_Update(self, #list, BUTTON_TOTAL, BUTTON_HEIGHT)
	
	local offset = FauxScrollFrame_GetOffset(self)
	
	for count = 1, BUTTON_TOTAL do
		local index = count + offset
		
		if index <= #list then
			buttons[count]:SetText(list[index])
			buttons[count]:Show()
		else
			buttons[count]:Hide()
		end
		
		
	end
end


function GUI()

	print("starting GUI experiment")
	
	if scrollFrame == nil then	
		scrollFrame = CreateFrame("ScrollFrame", "GIL_ScrollFrame", UIParent, "FauxScrollFrameTemplate")
	end

	scrollFrame:SetWidth  (BUTTON_WIDTH)
	scrollFrame:SetHeight (BUTTON_HEIGHT * BUTTON_TOTAL)
	scrollFrame:SetPoint  ("CENTER", UIParent)
	
	scrollFrame:SetScript("OnVerticalScroll",
	
		function(self, offset)
			FauxScrollFrame_OnVerticalScroll(self, offset, BUTTON_HEIGHT, GIL_UpdateList)
		end)

	for count = 1, BUTTON_TOTAL do
	
		buttons[count] = CreateFrame("Button", nil, scrollFrame:GetParent())
	
		if count == 1 then
			buttons[count]:SetPoint("TOP", scrollFrame)
		else
			buttons[count]:SetPoint("TOP", buttons[count - 1], "BOTTOM")
		end
	    
		buttons[count]:SetNormalFontObject("GameFontNormal")
		buttons[count]:SetSize(BUTTON_WIDTH, BUTTON_HEIGHT)
	end
	
	GIL_UpdateList(scrollFrame)
end

-------------
-- THE END --
-------------